<?php

class CCS_Catégorie extends CCS_Squelette{
	/**
	* Nombre TOTAL d'articles contenu dans la catégorie.
	* Le nombre d'articles chargé peut donc être inférieur au nombre TOTAL.
	* 
	* @var int
	*/
	private $totalArticles;
	
	/**
	* Tableau d'instances de CCS_Catégorie qui correspond aux sous-catégories.
	* 
	* @var array Liste des sous-catégories
	*/
	private $sousCatégories;
	
	/**
	* Tableau d'instances de CCS_Article qui correspond aux articles et directement rattachés à la catégorie en cours.
	* 
	* @var array Liste des articles
	*/
	private $articles;
	
	public function __construct($WP = 0, $avecSousCatégories = TRUE, $avecArticles = FALSE, $nbArticles = -1, $avecDoublons = FALSE, $orderby = "DESC"){
		$ID = 0;
		$titre = "";
		
		if(!is_int($WP)){
			$ID = $WP->cat_ID;
			$titre = $WP->cat_name;
			$this->totalArticles = $WP->count;
		}else $ID = $WP;
		
		parent::__construct($ID, $titre, $WP);
		
		$this->sousCatégories = array();
		$this->articles = array();
		
		//Chargement des données de la classe
		$this->charger($avecSousCatégories, $avecArticles, $nbArticles, $avecDoublons, $orderby);
	}
	
	/**
	* Chargement des sous catégories et des articles qui sont rattachés à la catégorie en cours.
	* 
	* @param bool $avecSousCatégories Faut-il charger les sous catégories ?
	* @param bool $avecArticles Faut-il charger les articles ?
	* @param int $nbArticles Si oui, combien d'articles ?
	* @param bool $avecDoublons Est-ce qu'on autorise à avoir le même article référencé dans plusieurs catégories ? (Si non, l'article sera rattaché à la sous catégorie de plus bas niveau).
	*/
	private function charger($avecSousCatégories, $avecArticles, $nbArticles, $avecDoublons, $orderby){
		$articleslus = array();
		
		//Chargement des éventuelles sous-catégories
		if($avecSousCatégories){
			foreach(get_categories(array("parent" => $this->ID, "hide_empty" => '0')) as $WP){
				//Nouvelle classe qui sera ensuite chargée
				$catégorie = new CCS_Catégorie($WP, $avecSousCatégories, $avecArticles, $nbArticles, $avecDoublons);
				
				//On récupère les ID des posts qui ont été chargés afin de ne plus avoir à les charger de nouveau si chargé dans une sous catégorie.
				$articleslus = array_merge($articleslus, $catégorie->getArticlesID());
				
				//Ajout de la catégorie dans notre tableau
				$this->sousCatégories[] = $catégorie;
			}
		}
		
		//Listing des articles rattachées à la catégorie
		if($avecArticles && $nbArticles > 0){
			$myposts = get_posts( array( 'showposts' => $nbArticles, 'orderby' => 'date', 'order' => $orderby, 'category' => $this->ID, 'post__not_in' => $articleslus ) );
			foreach ( $myposts as $qsdfq ){
				//Nouvelle instance d'Article
				$article = new CCS_Article($qsdfq, $this->ID);
				$this->articles[] = $article;
			}
			wp_reset_query();
		}
	}
	
	/**
	* Sert à récupérer tous les IDs des articles rattachés à cette catégorie
	* 
	* @return array Liste des IDs des articles.
	*/
	public function getArticlesID($avecRecursivité = FALSE){
		$array = array();
		
		foreach($this->getArticles($avecRecursivité) as $article){
			$array[] = $article->getID();
		}
		
		return $array;
	}
	
	/**
	* Retourne la liste de tous les articles de la catégorie en cours et des sous catégories (si demandé).
	* 
	* @param bool $avecRecursivité Faut-il aussi regarder dans les sous-catégories ?
	* 
	* @return array Liste de tous les articles
	*/
	public function getArticles($avecRecursivité = FALSE){
		$array = array();
		
		//On commence par récupérer les articles de la catégorie en cours
		foreach($this->articles as $article){
			$array[$article->getDate(FALSE)] = $article;
		}
		
		//On y ajoute celles des sous-catégories.
		if($avecRecursivité){
			foreach($this->sousCatégories as $catégorie){
				$array = array_merge($array, $catégorie->getArticles(TRUE));
			}
		}
		
		return $array;
	}
	
	/**
	* Dans la catégorie en cours, on chercher la sous-catégorie (récursivement ou non) souhaitée.
	* 
	* @param int $catID Numéro de la catégorie cherchée
	* @param bool $avecRecursivité Faut-il chercher aussi dans les sous-catégories des sous catégories ? (Defaut : Vrai)
	* @param bool $souscatégorieRacine Cherchons-nous la catégorie mère de la catégorie cherchée ou uniquement la catégorie cherchée ? (Defaut : Faux)
	* @param bool $départ Sert pour la récursivité, pour savoir si nous sommes une récursivité ou non.
	* 
	* @return CCS_Catégorie
	*/
	public function getCatégorie($catID, $avecRecursivité = TRUE, $souscatégorieRacine = FALSE, $départ = TRUE){
		$retour = NULL;
		
		/**
		* On retourne l'objet en cours si l'ID fourni correspond.
		* 
		* Ce cas ne peut être vrai que lors cette méthode n'a pas été appelée par elle-même.
		* En effet, le précédent appel de la méthode va d'abord regarder si l'ID ne correspondrait pas à ses sous-catégories.
		*/
		if($catID == $this->ID){
			
			//Si nous avons demandé une sous catégorie racine, on ne peux donc pas renvoyer l'objet en cours.
			if(!$souscatégorieRacine)$retour = $this;
			
		}else{
			
			//On regarde ensuite si l'ID ne correspondrait pas à celle d'une sous catégorie.
			$indice = $this->stdClassArray_search($this->sousCatégories,"ID", $catID);
			
			//Si c'est le cas, on retourne la sous catégorie
			if($indice >= 0){
				
				$retour = $this->sousCatégories[$indice];
				
			}else if($avecRecursivité){
				
				//On a demandé à regarder aussi dans les sous catégories ? Alors on regarde !
				foreach($this->sousCatégories as $catégorie){
					
					$retour = $catégorie->getCatégorie($catID, TRUE, $souscatégorieRacine, FALSE);
					
					if(!is_null($retour))break;
					
				}
				
			}
			
			/**
			* A ce stade, si nous nous apprêtons à retourner un objet, c'est que l'ID fourni a été trouvé.
			* Si nous avons demandé la sous catégorie racine, nous devons alors retourner l'objet en cours et non l'objet trouvé.
			*/
			if($souscatégorieRacine && !is_null($retour) && !$départ)$retour = $this;
			
		}
		
		return $retour;
	}
	
	/**
	* Permet de chercher une valeur dans un tableau d'objets en fournissant la valeur mais aussi l'attribut.
	* 
	* @param Array $tableau Tableau d'objets où effectuer la recherche
	* @param String $attribut nom de l'attribut où effectuer la recherche
	* @param Mixed $valeur valeur (quelque soit le type) que doit contenir l'attribut fourni
	* 
	* @return int number of the value found or -1
	*/
	private function stdClassArray_search($tableau, $attribut, $valeur){
		$id = -1;
		
		//On passe en revue tous les objets du tableau
		for($indice = 0; $indice < count($tableau); $indice++){
			$objet = $tableau[$indice];
			
			//Si un attribut de l'objet correspond à la valeur cherchée..
			if($objet->$attribut == $valeur){
				//On mémorise le numéro de l'élément dans le tableau.
				$id = $indice;
				
				//On quitte.
				break;
			}
		}
		
		return $id;
	}
}

?>